﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Common.Resources;
using HeuristicLab.Core.Networks;
using HeuristicLab.Core.Views;
using HeuristicLab.Networks.Programmable;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Visualization;

namespace HeuristicLab.Networks.Views.NetworkVisualization.Views {
  public class AddNodesChartMode : ChartMode {
    protected TypeSelectorDialog typeSelectorDialog;

    public override Image Image { get { return VSImageLibrary.RadialChart; } }
    public override string Text { get { return "Add &Nodes"; } }
    public override string ToolTipText { get { return "Add Nodes"; } }

    public AddNodesChartMode(ChartControl control) : base(control) { }

    public override void HandleOnMouseMove(object sender, MouseEventArgs e) {
      try {
        if (!(chartControl.Tag is IUserDefinedNetwork)) return;

        chartControl.RefreshPicture();

        var size = new Size(200, 100);

        using (var graphics = chartControl.CreatePictureGraphics())
        using (var pen = new Pen(Color.Gray) { DashStyle = DashStyle.Dash }) {
          graphics.DrawRectangle(pen,
            (int)Math.Round(e.X - size.Width * 0.5),
            (int)Math.Round(e.Y - size.Height * 0.5),
            size.Width,
            size.Height);
        }

      } finally {
        chartControl.UpdatePicture();
        base.HandleOnMouseMove(sender, e);
      }
    }

    public override void HandleOnMouseDown(object sender, MouseEventArgs e) {
      try {
        switch (e.Button) {
          case MouseButtons.Left:
            var network = chartControl.Tag as IUserDefinedNetwork;
            if (network == null) return;

            var node = CreateNode(network);
            if (node == null) return;

            var center = chartControl.Chart.TransformPixelToWorld(e.Location);
            var size = chartControl.Chart.TransformPixelToWorld(new Size(100, 50));
            var lowerLeft = center - new Offset(size);
            var upperRight = center + new Offset(size);
            node.VisualProperties = new NodeVisualProperties {
              LowerLeft = new Point2D<double>(lowerLeft.X, lowerLeft.Y),
              UpperRight = new Point2D<double>(upperRight.X, upperRight.Y)
            };
            network.Nodes.Add(node);
            break;
        }
      } finally {
        base.HandleOnMouseDown(sender, e);
      }
    }

    public override void HandleOnMouseLeave(object sender, EventArgs e) {
      chartControl.RefreshPicture();
      base.HandleOnMouseLeave(sender, e);
    }

    #region Helpers
    protected virtual INode CreateNode(INetwork network) {
      if (typeSelectorDialog == null) {
        typeSelectorDialog = new TypeSelectorDialog();
        typeSelectorDialog.Caption = "Select Node";
        typeSelectorDialog.TypeSelector.Caption = "Available Nodes";
        typeSelectorDialog.TypeSelector.Configure(typeof(INode), false, true);
      }

      if (typeSelectorDialog.ShowDialog(chartControl) == DialogResult.OK) {
        try {
          var n = (INode)typeSelectorDialog.TypeSelector.CreateInstanceOfSelectedType();
          n.Name = GetUniqueName(n.Name, network);
          return n;
        } catch (Exception ex) {
          ErrorHandling.ShowErrorDialog(chartControl, ex);
        }
      }
      return null;
    }

    protected virtual string GetUniqueName(string originalName, INetwork network) {
      if (!network.Nodes.ContainsKey(originalName))
        return originalName;

      string name;
      int index = 0;
      do {
        index++;
        name = originalName + index;
      } while (network.Nodes.ContainsKey(name));
      return name;
    }
    #endregion
  }
}
